/*
 *  Audit helper functions used throughout shadow-utils
 *
 *  Copyright (C) 2005, Red Hat, Inc.
 *
 *  Redistributable under the same terms as the other elements of
 *  shadow-utils v4.0.3.
 */

#include <config.h>

#ifdef WITH_AUDIT
#include <stdlib.h>
#include <syslog.h>
#include <stdarg.h>
#include <libaudit.h>
#include <errno.h>
#include <stdio.h>
int audit_fd;
#endif

void audit_help_open(void)
{
#ifdef WITH_AUDIT
	audit_fd = audit_open();
	if (audit_fd < 0) {
		/* You get these only when the kernel doesn't have
		 * audit compiled in. */
		if (errno == EINVAL || errno == EPROTONOSUPPORT ||
                               errno == EAFNOSUPPORT)
			return;
		fprintf(stderr, "Cannot open audit interface - aborting.\n");
		exit(1);
	}
#endif
}

/*
 * This function will log a message to the audit system using a predefined
 * message format. Parameter usage is as follows:
 *
 * type - type of message: AUDIT_USER_CHAUTHTOK for changing any account 
 *	  attributes.
 * pgname - program's name
 * op  -  operation. "adding user", "changing finger info", "deleting group"
 * name - user's account or group name. If not available use NULL.
 * id  -  uid or gid that the operation is being performed on. This is used
 *	  only when user is NULL.
 * result - 1 is "success" and 0 is "failed"
 */
void audit_logger(int type, const char *pgname, const char *op, 
	const char *name, unsigned int id, int result)
{
#ifdef WITH_AUDIT
        if (audit_fd < 0)
                return;
        else {
		char buf[PATH_MAX];
		const char *success;

		if (result)
			success = "success";
		else
			success = "failed";

		if (name)
			snprintf(buf, sizeof(buf), "%s: op=%s acct=%s res=%s",
				pgname, op, name, success);
		else
			snprintf(buf, sizeof(buf), "%s: op=%s id=%u res=%s",
				pgname, op, id, success);

		audit_send_user_message(audit_fd, type, buf);
        }
#endif
}

